<?php
   /*
   Admin.class.php (Atom PhotoBlog Version) - 

   A php-class distributed with AtomPhotoBlog.
   Copyright (C) 2007 by Sascha Tayefeh
   http://www.tayefeh.de/

   Smarty(TM) is not part of this code, but is maintained by 
   New Digital Group, Inc. and copyright 2001-2005 New Digital Group, Inc.
   See http://smarty.php.net/
 

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
   */

   /*!
   The Admin-class. 

   PARAMETERS:
   private $n // Number of times that the client has tried to login today.
   protected $accessGranted  // BOOL: Access granted=TRUE, Access denied=FALSE
   protected $remoteAddr // The adress of the client for whom the cookie was set
   public $cookieLifeTime // How long shall the cookie be stored?
   private $pw=array // Assoc. array that has to be set at construction time
   (
      "FILE"=>"./data/apbpw.php",   	// The path to the password file
      "FAILLOG"=>"./data/apb.log",  	// The path to the the logfile
      "SESSIONNAME"=>"apb",		// name of the session/cookies
      "LOGOUTURL"=>"", 		// Where to forward on successul login
      "LOGINURL=>"",	 		// Where to forward on successul logout
      "MAXTRIES"=>10,		// How many times can a single session try to login?
      "SALT"=>""			// The salt (randomly generated by install.php) for the password-hash
   );


   METHODS:
   private function countFailures()  // Count failed login-attempts
   protected function readPw() // Read password from password-file
   public function savePw($lg,$paw)  // save new Password and Login to file
   public function checkPw($lg,$paw)  // test if login and password are ok (errorcheck)
   public function getLoginName() // returns the login-name
   public function Admin($pw) // The constructor
   public function login() // Print login-form
   public function logout()  //  Destroy session and logout
   public function checkAccess() // Check if access is granted

   */

   class Admin
   {
      protected $accessGranted=FALSE; //! BOOL: Access granted=TRUE, Access denied=FALSE
      protected $remoteAddr="";  //! The adress of the client for whom the cookie was set
      public $cookieLifeTime=600000;  //! How long shall the cookie be stored?
      private $n=0;  //! Number of times that the client has tried to login today.

      private $pw=array
      (
	 "FILE"=>"./data/apbpw.php",
	 "FAILLOG"=>"./data/apb.log",
	 "SESSIONNAME"=>"apb",
	 "LOGOUTURL"=>"",
	 "LOGINURL"=>"",
	 "MAXTRIES"=>10,
	 "SALT"=>"uiwq4f8eu"
      );

      public function Admin($pw)
      {
	 foreach(array_keys($pw) as $key)
	 {
	    $this->pw[$key]=$pw[$key];
	 }
      }

      public function logout()
      {
	 $this->accessGranted=FALSE;
	 session_destroy();
	 return;
      }

      public function checkAccess()
      {
	 session_name ($this->pw['SESSIONNAME']); // Set the session-name
	 session_set_cookie_params($this->cookieLifeTime); // and its life-time
	 session_start();
	 if (!isset($_SESSION['passed']) || !$_SESSION['passed'])
	 {
	    $this->accessGranted=FALSE;
	 } else {
	    $this->accessGranted=TRUE;
	 }
	 return $this->accessGranted;
      }

      private function countFailures()
      {
	 $date[0]=date("Y-m-d",time()); // Get current date
	 $failFile=$this->pw['FAILLOG'];

	 if(!file_exists(($failFile)))
	 {
	    $fp=fopen($failFile,"w+") or die("***ERROR: Could not write admin logfile");
	    fclose($fp);
	 }

	 $fp    = fopen($failFile, "r")  or die("***ERROR: Could not read admin logfile"); // open failure log-file

	 while (!feof($fp))
	 {
	    $buffer= explode(";",fgets($fp, 500)); // import line and store in buffer-array
	    if(isset($buffer[1])) // Don't proceed blank lines
	    {
	       $date[1]=date("Y-m-d",$buffer[1]); // convert timeStamp from file

	       // Find out how many times this IP has failed to login today
	       if ($buffer[0] == $this->remoteAddr &&  $date[1]==$date[0])
	       {
		  $this->n++;
	       }
	    }
	 }
	 fclose($fp);
      }

      public function savePw($lg,$paw)
      {
	 if($this->accessGranted)
	 {
	    $pwfile=$this->pw['FILE'];
	    $fp=fopen($pwfile,"w+") or die("*** ERROR: Could not create password file ".$pwfile.". Check your directory and permission settings for the datadir");

	    $writeString="<?php\n";
	       $writeString.='$lg='.$lg;
	       $writeString.="\n";
	       $writeString.='$pw='.md5($this->pw['SALT'].$paw);
	       $writeString.="\n";
	    $writeString.='?>'.$lg;

	    fwrite($fp, $writeString);
	    fclose($fp);

	    //echo "<p class=\"apb form \" >Login was updated successfully.</p><p class=\"apb form\" >New Login: ".$lg."<br />New Password: ".$paw."</p>";
	 } else {
	    die("*** ERROR: Permission to change Login Data denied");
	 }
      }

      public function login()
      {
	 $this->remoteAddr=$_SERVER['REMOTE_ADDR'];// Get current remote ip
	 $this->countFailures(); // Get number of failures for this ip and for today
	 $pwfile=$this->pw['FILE'];
	 $img=array();

	 // Check if passwordfile exists. If not, create one with standard password
	 if(!file_exists(($pwfile)))
	 {
	    $img['fileExists']=FALSE;
	    return;
	 } else { $img['fileExists']=TRUE;}

	 // Deny Access, if failures exeed three for today
	 if($this->n >= $this->pw['MAXTRIES'])
	 {
	    $img['failExeeded']=TRUE;
	    $img['remoteAddr']=$this->remoteAddr;
	 } 
	 return $img;
      }

      protected function readPw()
      {
	 $pwfile=$this->pw['FILE'];

	 $fp=fopen($pwfile,"r") or die("***ERROR: Could not read password file");

	 while(! feof($fp))
	 {
	    $l=explode('=',fgets($fp, 1048576)); // import line and store in buffer-array
	    if(isset($l[0])) $key= trim(str_replace("$","",$l[0]));
	    if(isset($l[1])) $value= trim(preg_replace("/[;']/","",$l[1]));

	    if(isset($l[0]) && isset($l[1]))
	    {
	       $fileArray[$key]=$value;
	    }
	 }

	 return $fileArray;
      }

      public function getLoginName()
      {
	 $a=$this->readPw();
	 $lg=trim($a['lg']);
	 return $lg;
      }

      public function checkPw($lg,$paw)
      {
	 $failFile=$this->pw['FAILLOG'];
	 $this->remoteAddr=$_SERVER['REMOTE_ADDR'];
	 $this->countFailures(); // Get number of failures for this ip and for today
	 $img=array('failExeeded'=>FALSE,'wrongPasswd'=>FALSE);

	 if($this->n > $this->pw['MAXTRIES'])
	 {
	    $img['failExeeded']=TRUE;
	    $img['remoteAddr']=$this->remoteAddr;
	 } else
	 {
	    $pwfile = $this->readPw();

	    $pwpost['lg']=$lg;
	    $pwpost['pw']=md5($this->pw['SALT'].$paw);

	    if ( ($pwpost['lg'] != trim($pwfile['lg'])) || ($pwpost['pw'] != trim($pwfile['pw'])) )
	    {
	       $timeStamp=time();
	       $date=date("Y-m-d_H:i:s",$timeStamp);
	       $fp    = fopen($failFile, "a+")
	       or die("*** ERROR: Could not modify admin logfile file. Check your directory and permission settings for the datadir");

	       fwrite($fp, $this->remoteAddr.";".$timeStamp.";".$date.";".$_POST['username']."\n");
	       fclose($fp);

	       $img['wrongPasswd']=TRUE;
	       $img['remoteAddr']=$this->remoteAddr;
	    } else
	    {
	       $_SESSION['passed'] = true;
	    }
	 }
	 return $img;
      }
   }
?>
