<?
/*
   FeedRss.php adapted for atomPhotoBlog and distributed with AtomPhotoBlog.
   Version 0.5.3
   2007 by S. Tayefeh

   Implementation of Feed.interface.php for an Atom1.0 feed.
   
   This is a adaption for atomPhotoBlog. For a stand-alone version,
   please visit the downloads section of http://www.tayefeh.de/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

   */
require_once './apbClasses/Feed.interface.php';
require_once './apbClasses/codeCleaner.inc.php';

class FeedAtom implements Feed
{
	public $feedEncoding="utf-8";

	public $feedTitle="EmptyTitle";
	public $feedSubtitle="EmptsSubTitle";
	public $feedOwner="EmptyOwner";
	public $feedOwnerEmail="Nobody@nowhere.nocom";
	public $feedFile="atom.xml";
	public $feedFileUrl="http://localhost/atom.xml";
	public $feedRootUrl="http://localhost/"; 

	private $feedGeneratorVersion="1.0";
	private $feedGeneratorUrl="http://www.tayefeh.de";
	private $feedGenerator="FeedAtom.class by Sascha Tayefeh";

	private $isLoadFeed=FALSE;

	private $db;

	/**
	All properties can be set using an associative array as argument.
	*/
	public function FeedAtom($settingsArray=array())
	{
		foreach ($settingsArray as $key => $value)
		{
			$this->$key=$value;
		}

		$this->db= new DOMDocument('1.0', $this->feedEncoding);
		$this->db->formatOutput=true;
		$this->db->preserveWhiteSpace=false;
		$this->db->substituteEntities=false;

	}


	//! This method creates an unique Id for atom-feeds.
	private function createId($url)
	{
		$domain=str_replace("http://","",$url);
		$domain=substr_replace($domain,"",stripos($domain,'/'));
		$urlPath='/'.eregi_replace("http://[0-9a-zA-Z\-_+:\.#]*/",'',$url);
		$id ="tag:".$domain.",".date("Y-m-d",time()).":";
		$id.=$urlPath."/".date("YmdHis");
		return $id.rand(1,1000);
	}



	/**
	* Create the AtomHeader and save it.
	*
	**/
	public function initFeedFile()
	{

		// This document is local since it is just temporary
		$db= new DOMDocument('1.0', $this->feedEncoding);
		$db->formatOutput=true;
		$db->preserveWhiteSpace=true;
		$db->substituteEntities=true;

		//  --------------------
		//    Create Atom Feed
		//  --------------------

		// Header Begin
		$feed=$db->appendChild($db->createElement('feed'));
		$feed->setAttribute('xmlns','http://www.w3.org/2005/Atom');

		$title=$feed->appendChild($db->createElement('title',xmlRecode(utf8_encode($this->feedTitle))));
		$title->setAttribute('type','text');

		$linkAlt=$feed->appendChild($db->createElement('link'));
		$linkAlt->setAttribute('rel','alternate');
		$linkAlt->setAttribute('type','text/html');
		//	 $linkAlt->setAttribute('hreflang','de');
		$linkAlt->setAttribute('href',$this->feedRootUrl);

		$linkSelf=$feed->appendChild($db->createElement('link'));
		$linkSelf->setAttribute('rel','self');
		$linkSelf->setAttribute('type','application/atom+xml');
		$linkSelf->setAttribute('href',$this->feedFileUrl);

		$subTitle=$feed->appendChild($db->createElement('subtitle',$this->feedSubtitle));
		$subTitle->setAttribute('type','text');

		$update=$feed->appendChild($db->createElement('updated',date("c",time())));

		$author=$feed->appendChild($db->createElement('author'));
		$author->appendChild($db->createElement('name',xmlRecode(utf8_encode($this->feedOwner))));

		$id=$this->createId($this->feedRootUrl);
		$id=$feed->appendChild($db->createElement('id',$id));

		$rights=$feed->appendChild($db->createElement('rights','Copyright (c) '.date("Y",time()).', '.xmlRecode(utf8_encode($this->feedOwner))));

		$generator=$feed->appendChild($db->createElement('generator',xmlRecode(utf8_encode($this->feedGenerator))));
		$generator->setAttribute('uri',xmlRecode(utf8_encode($this->feedGeneratorUrl)));
		$generator->setAttribute('version',$this->feedGeneratorVersion);

		$db->save($this->feedFile);
	}

	public function createEntry($entryData=array())
	{

		$myIds=NULL;
		$keepAtomId=$entryData['keepAtomId'];

		//  -------------- Create atom.xml entry
		if(!$this->isLoadFeed) $this->readFeedFile();
		$feed=$this->db->documentElement;

		// Entry Begin

		// Distinguish between first entry and further entries. We need to insert
		// new element atop rather than to append.
		$myIds=$feed->getElementsByTagName('id');
		if($myIds->length>1)
		{
			$refNode=$myIds->item(1)->parentNode;
			$entry=$refNode->parentNode->insertBefore($this->db->createElement('entry'),$refNode);
		}
		else
		$entry=$feed->appendChild($this->db->createElement('entry'));

		$eTitle=$entry->appendChild($this->db->createElement('title',utf8_encode(htmlRecodeLight($entryData['itemTitle']))));
		$eTitle->setAttribute('type','text');

		$eAuthor=$entry->appendChild($this->db->createElement('author'));
		$eAuthor->appendChild($this->db->createElement('name',xmlRecode(utf8_encode($this->feedOwner))));

		$eLinkAlt=$entry->appendChild($this->db->createElement('link'));
		$eLinkAlt->setAttribute('href',$entryData['link']);
		$eLinkAlt->setAttribute('rel','alternate');
		$eLinkAlt->setAttribute('type','text/html');

		$eLinkEncl=$entry->appendChild($this->db->createElement('link'));

		if(isset($entryData['enclosureUrl']) && $entryData['enclosureUrl']!=NULL)
		{
		   $eLinkEncl->setAttribute('href',$entryData['enclosureUrl']);
		   $eLinkEncl->setAttribute('length',$entryData['enclosureSize']);
		   $eLinkEncl->setAttribute('rel','enclosure');
		   $eLinkEncl->setAttribute('type',$entryData['enclosureMimeType']);
		}

		// TODO: replace link to imgUrl by something more appropiate
		if($keepAtomId) $eId=$entry->appendChild($this->db->createElement('id',$entryData['oldAtomId']));
		else $eId=$entry->appendChild($this->db->createElement('id',$this->createId($entryData['imgUrl'])));

		$eUpdated=$entry->appendChild($this->db->createElement('updated',date("c",time())));

		$eRights=$entry->appendChild($this->db->createElement('rights','Copyright (c) '.date("Y",time()).', '.xmlRecode(utf8_encode($this->feedOwner))));

		$eSummary=$entry->appendChild($this->db->createElement('summary',utf8_encode(htmlRecodeLight($entryData['description']))));
		$eSummary->setAttribute('type','text');

		$eContent=$entry->appendChild($this->db->createElement('content'));
		$eContent->setAttribute('type','xhtml');
		$eContentDiv=$eContent->appendChild($this->db->createElement('div'));
		$eContentDiv->setAttribute('xmlns','http://www.w3.org/1999/xhtml');
		$eContentDivA=$eContentDiv->appendChild($this->db->createElement('a'));
		$eContentDivA->setAttribute('href',$entryData['link']);
		$eContentDivAImg=$eContentDivA->appendChild($this->db->createElement('img'));
		$eContentDivAImg->setAttribute('src',$entryData['imgUrl']);
		$eContentDivAImg->setAttribute('alt',xmlRecode(utf8_encode(htmlRecodeLight($entryData['itemTitle']))));

		$eContentDivP=$eContentDiv->appendChild($this->db->createElement('p',utf8_encode(xmlRecode($entryData['description']))));

		// Update feed's modification date
		$this->db->getElementsByTagName('updated')->item(0)->nodeValue=date("c",time());

		// Entry END
		$this->db->save($this->feedFile);
		return;
	}

	public function deleteEntry($query)
	{
	   $status=TRUE;

	   $this->readFeedFile();
	   $feed=$this->db->documentElement;

	   $xpath = new DOMXPath($this->db);
	   $xpath->registerNamespace('pf', 'http://www.w3.org/2005/Atom');

	   $entries = $xpath->query($query);

	   $nEntries=$entries->length;
//	   echo "<br/>$query<br/>Atom Entries found: ".$nEntries;

	   if($nEntries==1) $feed->removeChild($entries->item(0)->parentNode);
	   else $status=FALSE;

	   $this->db->save($this->feedFile);
	   return $status;
	}

	/**
	*
	* @param $query This is a valid XPath-Query that identifies the entry to be updated
	* @param $newEntryData The data-array for FeedAtom::createEntry($data)
	*/
	public function updateEntry($query,$newEntryData=array())
	{
	   $status=TRUE;

	   if(!$this->isLoadFeed) $this->readFeedFile();
	   $feed=$this->db->documentElement;

	   $xpath = new DOMXPath($this->db);
	   $xpath->registerNamespace('pf', 'http://www.w3.org/2005/Atom');
	   $entries = $xpath->query($query);

	   $nEntries=$entries->length;
	   if($nEntries==1)
	   {
	      $parentNode=$entries->item(0)->parentNode;
	      // Remember old id
	      $newEntryData['oldAtomId']=$parentNode->getElementsByTagName('id')->item(0)->nodeValue;

	      // Remember mime-type
	      $newEntryData['enclosureMimeType']=$entries->item(0)->getAttribute('type');

	      // Remove old entry
	      $feed->removeChild($parentNode);

	      // Create Atom Entry
	      $this->createEntry($newEntryData);
	   }
	   else $status=FALSE;

	   return $status;

	}

	/*!
	Read the Atom1.0 file. If it doesn't exist, create it!
	*/
	public function readFeedFile()
	{
	   if(!file_exists($this->feedFile))  $this->initFeedFile();
	   $this->db->load($this->feedFile) or die("Could not load atom-file");
	   $this->isLoadFeed=TRUE;
	}
} 
?>
