<?
 /*
   FeedRss.php adapted for atomPhotoBlog and distributed with AtomPhotoBlog.
   Version 0.5.3
   2007 by S. Tayefeh

   Implementation of Feed.interface.php for an Rss2.0 feed.
   
   This is a adaption for atomPhotoBlog. For a stand-alone version,
   please visit the downloads section of http://www.tayefeh.de/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

   */
require_once './apbClasses/Feed.interface.php';
require_once './apbClasses/codeCleaner.inc.php';

class FeedRss implements Feed
{
	public $feedEncoding="utf-8";

	public $feedTitle="EmptyTitle";
	public $feedSubtitle="EmptsSubTitle";
	public $feedOwner="EmptyOwner";
	public $feedOwnerEmail="Nobody@nowhere.nocom";
	public $feedFile="rss.xml";
	public $feedFileUrl="http://localhost/rss.xml";
	public $feedRootUrl="http://localhost/"; 

	private $feedGeneratorVersion="1.0";
	private $feedGenerator="FeedRss.class by Sascha Tayefeh";

	private $isLoadFeed=FALSE;

	private $db;

	/**
	All properties can be set using an associative array as argument.
	*/
	public function FeedRss($settingsArray=array())
	{
		foreach ($settingsArray as $key => $value)
		{
			$this->$key=$value;
		}

		$this->db= new DOMDocument('1.0', $this->feedEncoding);
		$this->db->formatOutput=true;
		$this->db->preserveWhiteSpace=false;
	}

	/**
	* Create the RssHeader and save it.
	* To use this function, you must initialize:
	*
	**/
	public function initFeedFile()
	{

		// This document is local since it is just temporary
		$db= new DOMDocument('1.0', $this->feedEncoding);
		$db->formatOutput=true;
		$db->preserveWhiteSpace=true;
		$rss=$db->appendChild($db->createElement('rss'));
		$rss->setAttribute('xmlns:content','http://purl.org/rss/1.0/modules/content/');
		$rss->setAttribute('version','2.0');

		$channel=$rss->appendChild($db->createElement('channel'));

		$title=$channel->appendChild($db->createElement('title',xmlRecode(utf8_encode($this->feedTitle))));
		$link=$channel->appendChild($db->createElement('link',xmlRecode(utf8_encode($this->feedRootUrl))));
		$description=$channel->appendChild($db->createElement('description',xmlRecode(utf8_encode($this->feedSubtitle))));

		$lastBuildDate=$channel->appendChild($db->createElement('lastBuildDate',date("r",time())));
		$pubDate=$channel->appendChild($db->createElement('pubDate',date("r",time())));
		$copyright=$channel->appendChild($db->createElement('copyright','Copyright (c) '.date("Y",time()).', '.xmlRecode(utf8_encode($this->feedOwner))));
		$docs=$channel->appendChild($db->createElement('docs',utf8_encode($this->feedFileUrl)));
		$generator=$channel->appendChild($db->createElement('generator',xmlRecode(utf8_encode($this->feedGenerator." V".$this->feedGeneratorVersion))));
		$managingEditor=$channel->appendChild($db->createElement('managingEditor',xmlRecode(utf8_encode($this->feedOwnerEmail))));
		$webMaster=$channel->appendChild($db->createElement('webMaster',xmlRecode(utf8_encode($this->feedOwnerEmail))));
		$db->save($this->feedFile);
	}

	public function createEntry($entryData=array())
	{

		$myFirstChild=NULL;

		if(!$this->isLoadFeed) $this->readFeedFile();

		$rss=$this->db->documentElement;

		$channel=$this->db->getElementsByTagName("channel")->item(0);

		// Distinguish between first entry and further entries. We need to insert
		// new element atop rather than to append.
		$myFirstChild=$channel->getElementsByTagName("item");
		if($myFirstChild->length)
		$item=$channel->insertBefore($this->db->createElement('item'),$myFirstChild->item(0));
		else
		$item=$channel->appendChild($this->db->createElement('item'));

		$title=$item->appendChild($this->db->createElement('title',utf8_encode(htmlRecodeLight($entryData['itemTitle']))));

		$link=$item->appendChild($this->db->createElement('link',utf8_encode($entryData['link'])));

		$pubDate=$item->appendChild($this->db->createElement('pubDate',date("r",time())));

		$guid=$item->appendChild($this->db->createElement('guid',xmlRecode(utf8_encode($entryData['link']))));
		$guid->setAttribute('isPermaLink','true');

		$description=$item->appendChild($this->db->createElement('description'));
		$description->nodeValue=(utf8_encode(htmlRecodeLight($entryData['description'])));
		$content=$item->appendChild($this->db->createElement('content:encoded'));
		$CDATA=$content->appendChild($this->db->createCDATASection( $entryData['cDataString']));
		$this->db->getElementsByTagName('lastBuildDate')->item(0)->nodeValue=date("r",time());
		$this->db->getElementsByTagName('pubDate')->item(0)->nodeValue=date("r",time());
		$this->db->save($this->feedFile);
	}

	public function deleteEntry($query)
	{
	      $status=TRUE;

	      if(!$this->isLoadFeed) $this->readFeedFile();
	      $rss=$this->db->documentElement;

	      $xpath = new DOMXPath($this->db);

	      // A valid XPath query
	      $entries = $xpath->query($query);

	      if($entries->length)
	      {
		      $nEntries=$entries->length;
		      $parentNode=$entries->item(0)->parentNode->parentNode;
		      if($nEntries==1) $parentNode->removeChild($entries->item(0)->parentNode);
		      else $status=FALSE;
		      $this->db->save($this->feedFile);
	      } else
	      {
		      $status=FALSE;
	      }
	      return $status;
	}

	/**
	*
	* @param $query This is a valid XPath-Query that identifies the entry to be updated
	* @param $newEntryData The data-array for FeedRss::createEntry($data)
	*/
	public function updateEntry($query,$newEntryData=array())
	{
	      $status=TRUE;

	      if(!$this->isLoadFeed) $this->readFeedFile();
	      $rss=$this->db->documentElement;

	      $xpath = new DOMXPath($this->db);
	      $entries = $xpath->query($query);

	      // If query was successful (i.e. 1 or more items found)
	      if($entries->length)
	      {
		      $nEntries=$entries->length;
		      $parentNode=$entries->item(0)->parentNode->parentNode;

		      if($nEntries==1) {
			      // If exactly one item was found, proceed
			      if(!$parentNode->removeChild($entries->item(0)->parentNode))
			      die('COULD NOT REMOVE ITEM FROM RSS-FILE');
		      }
		      else 
		      {
			      $status=FALSE; // If more than one item was found
			      if(!$status) die('COULD NOT REMOVE ITEM FROM RSS-FILE because more than ONE item was identified');
		      }

		      $this->createEntry($newEntryData);
	      } else
	      {
		      $status=FALSE;
	      }
	      return $status;
	}

	/*!
	Read the Rss2.0 file. If it doesn't exist, create it!
	*/
	public function readFeedFile()
	{
		if(!file_exists($this->feedFile)) $this->initFeedFile();
		$this->db->load($this->feedFile) or die("Could not load rss-file");
		$this->isLoadFeed=TRUE;
	}


} 
?>
