<?php

   /*
   resizeImage.inc.php - Distributed with AtomPhotoBlog
   Version 1.2
   2007 by S. Tayefeh

   This file provides a single function that resizes a jpeg or a png image.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

   */



   /**
   This is the image resizer function. Its arguments are self-explaining.
   It returns a pointer to the new image, so it can be saved or something.
   */
   function resizeImage
   (
      $uploadedClientFileName,
      $serverTempFileName,
      $imageMaxWidth = 850,
      $imageMaxHeight = 600,
      $quality=90,
      $embedCopyright=TRUE,
      $owner=""
   )
   {
      preg_match("/(\.\w+)$/",$uploadedClientFileName,$match); $uploadedFileType = strtolower($match[1]);

      if(!in_array($uploadedFileType, array(".jpg",".jpeg",".png")))
      die ("<p class=\"apb form error\">*** Error: File Format $uploadedFileType NOT SUPPORTED</p><p class=\"apb form\">Currently, only JPEG and PNG files are supporten.</p>");

      switch ($uploadedFileType)
      {
	 case ".jpg":
	 case ".jpeg":
	    $image = imagecreatefromjpeg($serverTempFileName);
	    break;
	 case ".png":
	    $image = imagecreatefrompng($serverTempFileName);
	    break;
	 default:
	    break;
      }

      list($oldWidth, $oldHeight) = getimagesize($serverTempFileName);
      if (($oldWidth > $imageMaxWidth) || ($oldHeight > $imageMaxHeight))
      {
	 $widthRatio = $oldWidth / $imageMaxWidth;
	 $heightRatio = $oldHeight / $imageMaxHeight;
	 if ($widthRatio >= $heightRatio)
	 {
	    $newWidth = $imageMaxWidth;
	    $newHeight = (int) ($oldHeight / $widthRatio);
	 }
	 else
	 {
	    $newHeight = $imageMaxHeight;
	    $newWidth = (int) ($oldWidth / $heightRatio);
	 }
	 $newImage = @imagecreatetruecolor($newWidth, $newHeight);
	 imagecopyresampled($newImage, $image, 0, 0, 0, 0, $newWidth, $newHeight, $oldWidth, $oldHeight);
      }
      else
      {
	 $newImage = $image;
      }

      // Embbed copyright into highRes pictures
      if($embedCopyright)
      {
	 $string= "Copyright ".date("Y",time())." by ".$owner;
	 $gray = imagecolorallocate($newImage,200,200,200);
	 $darkgray = imagecolorallocate($newImage,40, 40, 40);
	 $px = 5;
	 $py = (imagesy($newImage) - 14);
	 imagefilledrectangle ( $newImage, 0, $py, 250, ($py+20), $darkgray);
	 imagestring($newImage, 2, $px, $py, $string, $gray);

      }
      // And return the pointer to the image (I hope, it's just the pointer ...)
      return $newImage;
   }
?>
